import re
from collections import defaultdict as ddict
from readers.adeireader import ADEIReader

class DBAdapter(object):
    
    regex_sensor = re.compile('(\w+).(\w+)')
    
    def __init__(self, config, dbreader):                                                                           
        self._url = config.get('url')                                           
        self._server = config.get('server')                                     
        self._dbname = config.get('database')
        self._dbreader = dbreader
        
    def parse_sensors(self, sensors):
        res = ddict(list)
        for sensor in sensors:
            m = self.regex_sensor.search(sensor)
            if m:
                groupname = m.group(1)
                sensorname = m.group(2)
                res[groupname].append(sensorname)
        return res

    def read_from_source(self, group, sensors):
        pass
        
    def read_sensors(self, sensors, group=''):
        res = {}
        if not group:
            sensors = self.parse_sensors(sensors)
            for g, v in sensors.iteritems():
                res[g] = self.read_from_source(g, v)
        else:
            res[group] = self.read_from_source(group, sensors)
        return res
    
    def get_sensor_list(self):
        return None

    @property
    def sensors(self):
        return self.get_sensor_list()                                             

                                                                                
class ADEIAdapter(DBAdapter):
    
    def __init__(self, config, dbreader=None):
        super(ADEIAdapter, self).__init__(config, dbreader)
        self._dbreader = ADEIReader(self._url, self._server, self._dbname)

    def get_sensor_list(self):
        return self._dbreader.sensors

    def read_from_source(self, group, sensors):
        return { d[0]:d[1] for d in self._dbreader.getdata(group, *sensors) }
        # data = {}
        # for g, v in sensors.iteritems():
        #     data[g] =  { d[0]:d[1] for d in self._dbreader.getdata(g, *v) }
        # return data